(* e+e- -> e+e- *)

PrepareIndex[mu, nu]

{to, from} = MandelstamRules[{p1, p2, p3, p4}, {0, 0, 0, 0}, s, t, u]

mm[l1_, l2_] := mm[l1, l2] = (* The matrix element, with l1 and l2 being
				the helicity of the incoming electron and 
				positron *)
	SpinorUbar[p3] ** 
	Vertex[Electron, Electron, {Photon, p1-p3, mu}] ** SpinorU[p1, l1] *
     	SpinorVbar[p2, l2] **
	Vertex[Electron, Electron, {Photon, p1-p3, nu}] ** SpinorV[p4] *
	Propagator[{Photon, p1-p3, mu, nu}] -
     	SpinorUbar[p3] **
	Vertex[Electron, Electron, {Photon, p1+p2, mu}] ** SpinorV[p4] *
     	SpinorVbar[p2, l2] ** 
	Vertex[Electron, Electron, {Photon, p1+p2, nu}] ** SpinorU[p1, l1] *
	Propagator[{Photon, p1+p2, mu, nu}]

me2[l1_, l2_] := me2[l1, l2] = (*	The matrix element squared,
					contracting over the lorentz indices.
					The replacement p4->p1+p2-p3 is
					necessary to get rid of the
					Eps[p1, p2, p3, p4] term. *)
     Factor[Contract[
	AbsSquared[mm[l1, l2]], {mu, Conjugate[mu], nu, Conjugate[nu]}] /.
		p4->p1+p2-p3]

{pp1, pp2} = Decay[{0, 0, 0, Sqrt[s]}, {1, 0}, {0, 0}] (* Incoming momenta *)

cc[l1_, l2_] := cc[l1, l2] = (* Unevaluated PhaseSpaceIntegral.
				We replace the (unique) integration variable
				by cth, and remove the cth integration. *)
				
	Block[{cs = CrossSection[me2[l1, l2],
			p1 -> pp1, p2 -> pp2, Cylindrical[p3,p4]], xr, x},
	xr = cs[[4, 1]]; x = First[xr];
	cs /. {a___, xr, b___} -> {a, b} /. x -> cth]


dcs[l1_, l2_] := dcs[l1, l2] = (* Actually evaluating the
				  PhaseSpaceIntegral *)
				Factor[EvaluatePhaseSpaceIntegral[cc[l1, l2]]]
						

pols = {{Left, Left}, {Left, Right}} (* The {Right, _ } combinations
					are related to those by symmetry *)


(* result should be

               2              2      4           2
          Alpha  Pi (1 + 6 cth  + cth )   8 Alpha  Pi
         {-----------------------------, -------------}
                            2                      2
                  (-1 + cth)  s          (-1 + cth)  s

*)

result = Apply[dcs, pols, 1]
