(* Compton Scattering *)

Clear[p1, p2, p3, p4, s, t, u, me, mu, nu, SpaceTimeDimension]

PrepareIndex[mu, nu]

SetMass[{Electron, me}]

SetMandelstam[{p1, p2, p3, p4}, {me, 0, me, 0}, s, t, u]

mm = MasslessVectorPolarization[p2, mu] MasslessVectorPolarization[p4, nu] *
	SpinorUbar[p3] ** 
	(Vertex[Electron, Electron, {Photon, p4, nu}] **
	Propagator[{Electron, p1 + p2}] ** 
	Vertex[Electron, Electron, {Photon, p2, mu}] +
	Vertex[Electron, Electron, {Photon, p2, mu}] **
	Propagator[{Electron, p1 - p4}] **
	Vertex[Electron, Electron, {Photon, p4, nu}]) ** SpinorU[p1]

(* To simplify matters we will set the electron mas to zero here *)

me = 0

me2 = Together[Expand[
	Contract[AbsSquared[mm], {mu, nu, Conjugate[mu], Conjugate[nu]}]]]

(* Note this may take a little while *)

me2 = Factor[me2 /. t->(2 me^2 -s-u)]

cc = CrossSection[me2,
	p1->{0, 0, Sqrt[s]/2, Sqrt[s]/2}, p2->{0, 0, -Sqrt[s]/2, Sqrt[s]/2},
	Cylindrical[p3,p4], Mandelstam -> {u}]

(* Need to put in a cut because of the t-channel divergence due to
zero electron mass *)

cc = cc /. {a_, -1, 1} :> {a, -cut, cut}

(* The result should be

        -((Alpha  Pi (-2 + SpaceTimeDimension)

         (-14 cut + 3 SpaceTimeDimension cut +

                                             -1 + cut
           (-4 + 2 SpaceTimeDimension) Log[-(--------)])) / s)
                                             1 + cut
*)

result = Collect[#, Log]& /@ 
   (Factor[EvaluatePhaseSpaceIntegral[cc]] /. Log[a_] :>
    Log[Factor[a]] //. a_ Log[b_] + c_ Log[d_] :> a Log[b/d] /; a + c === 0) 
